--[[
	Copyright (C) 2016  Fuhrbolg - Hyjal

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>

	Author's note: Under no circumstances should this Addon be used under any situation.
	It is dangerous and will cause significant harm to your computer. You have been warned.
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("MongoMon")

-- Applied to the background textures
ALPHA_VALUE = 0.40

-- These are the two primary backgrounds for our scoreboard. The only difference is the border texture
-- which becomes more prominent when the player is in the top 5 killing blows. The gradient change 
-- occurs elsewhere and for different reasons.
BACKDROP_RED = {
	bgFile = "Interface\\AddOns\\MongoMon\\Res\\RedAlphaBackground", 
	edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
	edgeSize = 30,
	insets = { left = 8, right = 8, top = 8, bottom = 8 }
}

BACKDROP_GREEN = {
	bgFile = "Interface\\AddOns\\MongoMon\\Res\\GreenAlphaBackground", 
	edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
	edgeSize = 30,
	insets = { left = 8, right = 8, top = 8, bottom = 8 }
}

BACKDROP_BAD = {
	bgFile = "Interface\\AddOns\\MongoMon\\Res\\BlackAlphaBackground", 
	edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
	edgeSize = 30,
	insets = { left = 8, right = 8, top = 8, bottom = 8 }
}

--[[
 -	Adds battleground stats to the MatchHistoryFrame. This is done programatically
 -	because there are numerous battlegrounds and doing it manually is too boring.
 -
 -	@param Frame parent
--]]
function createBattlegroundStats(parent)
	local i = 0
	local width = 125
	local height = 50
	
	-- Build one entry for each BG map.
	for mapId, mapName in pairs(BG_MAP_IDS) do
		local offsetX = (i / 5 > 1 and  (i - 6) * width or (i * width)) + 50
		local offsetY = (i / 5 > 1 and -height - 15 or 0) - 10
		local frame = CreateFrame("FRAME", nil, parent)
		frame:SetPoint("TOPLEFT", parent, "TOPLEFT", offsetX, offsetY)
		frame:SetSize(width, height)
		
		frame.titleFontString = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		frame.titleFontString:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0) 
		frame.titleFontString:SetJustifyH("LEFT")
		frame.titleFontString:SetText(mapName)
		
		frame.iconTextureContainer = CreateFrame("FRAME", nil, frame)
		frame.iconTextureContainer:SetPoint("TOPLEFT", frame.titleFontString, "BOTTOMLEFT", 0, -5)
		frame.iconTextureContainer:SetSize(35, 35)
		
		frame.iconTexture = frame.iconTextureContainer:CreateTexture()
		frame.iconTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
		frame.iconTexture:SetAllPoints()

		frame.winLossFontString = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
		frame.winLossFontString:SetPoint("TOPLEFT", frame.iconTextureContainer, "TOPRIGHT", 15, -3) 
		frame.winLossFontString:SetJustifyH("CENTER")
		frame.winLossFontString:SetText("0 - 0")
		
		frame.winRateFontString = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
		frame.winRateFontString:SetPoint("TOPLEFT", frame.iconTextureContainer, "TOPRIGHT", 15, -20) 
		frame.winRateFontString:SetJustifyH("CENTER")
		frame.winRateFontString:SetText("")
		
		parent[mapName] = frame
		i = i + 1
	end
end

--[[
 -	Updates the battleground stats section of the match history report.
 -
 -	@param Frame parent UI elemetn
 -	@param table mapData
--]]
function updateBattlegroundStats(parent, mapData)
	-- Reset all BG stats to default because they might no longer have data.
	for mapId, mapName in pairs(BG_MAP_IDS) do
		local frame = parent[mapName]
		frame.iconTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
		frame.iconTexture:SetAlpha(ALPHA_VALUE)
		frame.winLossFontString:SetText("0 - 0")
		frame.winLossFontString:SetFontObject("GameFontDisableSmall")
		frame.winRateFontString:SetText("")
	end
	
	-- Populate all BG frames with fresh stats of power.
	for mapName, bgData in pairs(mapData) do
		local frame = parent[mapName]
		frame.iconTexture:SetAlpha(1)
		frame.winLossFontString:SetFontObject("GameFontHighlightSmall")
		if bgData.winRate >= 50 then
			frame.iconTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestCheck")
		else
			frame.iconTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestX")
		end
		frame.winLossFontString:SetText(bgData.wins .. " - " .. bgData.losses)
		frame.winRateFontString:SetText(round(bgData.winRate, 1) .. "%")
	end
end

--[[
 -	Adds top specs to a section of the after action report. There's a total of 12 of
 -	these so its just makes more sense to build them here.
 -
 -	@parent Frame to be embedded in
--]]
function createTopSpecs(parent) 
	parent.specFrame1 = CreateFrame("FRAME", nil, parent)
	parent.specFrame1:SetPoint("TOP", parent, "TOP", -80, -20)
	
	parent.specFrame2 = CreateFrame("FRAME", nil, parent)
	parent.specFrame2:SetPoint("TOP", parent, "TOP", 0, -20)
	
	parent.specFrame3 = CreateFrame("FRAME", nil, parent)
	parent.specFrame3:SetPoint("TOP", parent, "TOP", 80, -20)
	
	local frames = { parent.specFrame1, parent.specFrame2, parent.specFrame3 }
	for key, frame in pairs(frames) do
		frame:SetBackdrop(BACKDROP_RED)
		frame:SetWidth(45)
		frame:SetHeight(45)
		
		frame.specFrameTextureContainer = CreateFrame("FRAME", nil, frame)
		frame.specFrameTextureContainer:SetPoint("CENTER", frame, "CENTER", 0, 0)
		frame.specFrameTextureContainer:SetSize(30, 30)
		
		frame.specFrameTexture = frame.specFrameTextureContainer:CreateTexture()
		frame.specFrameTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\GreenAlphaBackground")
		frame.specFrameTexture:SetAllPoints()

		frame.titleFontString = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		frame.titleFontString:SetPoint("TOP", frame, "BOTTOM", 0, 0) 
		frame.titleFontString:SetJustifyH("CENTER")
		frame.titleFontString:SetText("")

		frame.classFontString = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
		frame.classFontString:SetPoint("TOP", frame.titleFontString, "BOTTOM", 0, -2) 
		frame.classFontString:SetJustifyH("CENTER")
		frame.classFontString:SetText("")		
	end
end

--[[
 -	Updates the top spec icons and values.
 -
 -	@parent Frame to be embedded in
 -	@param table players Contains three player objects with class, spec, and percent, number
 -	@param boolean Whether to display the percent value in the player object
--]]
function updateTopSpecs(parent, players, showPercent)
	-- Hide just in case there arent all three top specs
	for _,i in pairs({1, 2, 3}) do
		local frame = parent["specFrame" .. i]
		frame.specFrameTexture:Hide()
		frame.titleFontString:Hide()
		frame.classFontString:Hide()
	end
	
	for key, player in pairs(players) do
		local frame = parent["specFrame" .. key]
		frame.specFrameTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\SpecIcons")
		frame.specFrameTexture:SetTexCoord(unpack(SPEC_ICONS[player.class .. "-" .. player.spec]))
		if showPercent then
			frame.titleFontString:SetText("#" .. key .. " (" .. round(player.percent, 0) .. "%)")
		else
			frame.titleFontString:SetText("#" .. key)
		end
		frame.classFontString:SetText(player.class .. "\n" .. player.spec)
		frame.classFontString:Show()
		frame.specFrameTexture:Show()
		frame.titleFontString:Show()
	end
end

--[[
 -	Creates a Legend for a Pie chart since this isn't provided by
 -	LibGraph-2.0. So we will have to create a little frame. Provided
 -	is a table that contains a legend value and a color.
 -
 -	@param string name of Frame
 -	@param Frame parent frame
 -	@param string relativePosition from this frame to parent
 -	@param string parentPosition Position to parent
 -	@return Frame 
--]]
function createPieLegendFrame(name, parent, relativePosition, parentPosition, values)
	local legendFrame = CreateFrame("FRAME", name, parent)
	legendFrame:SetWidth(100)
	legendFrame:SetHeight(25)
	legendFrame:SetPoint(relativePosition, parent, parentPosition, 0, 10)
	local lastFontString
	for key, color in pairs(values) do
		local fontString = legendFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		if not lastFontString then
			fontString:SetPoint("LEFT", legendFrame, "LEFT", 0, 0) 
		else
			fontString:SetPoint("LEFT", lastFontString, "RIGHT", 3, 0) 
		end
		if key == "ranked" then
			fontString:SetText(L["Ranked"])
		else
			fontString:SetText(L["Unranked"])
		end
		fontString:SetTextColor(color[1], color[2], color[3], 1)
		lastFontString = fontString
	end
end

--[[
 -	Builds the title for the line chart that displays under the chart
 -	Should just print some simple text.
 -
 -	@param string name of Frame
 -	@param Frame parent frame
 -	@param string relativePosition from this frame to parent
 -	@param string parentPosition Position to parent
 -	@param string categoryName will appear in Legend title
 -	@return Frame
--]]
function createLineLegend(name, parent, relativePosition, parentPosition, categoryName)
	local legendFrame = CreateFrame("FRAME", name, parent)
	legendFrame:SetWidth(100)
	legendFrame:SetHeight(25)
	legendFrame:SetPoint(relativePosition, parent, parentPosition, 0, -3)
	
	local playerFontString = legendFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	playerFontString:SetPoint("LEFT", legendFrame, "LEFT", -25, 0) 
	playerFontString:SetTextColor(0, 1, 0, 1)
	playerFontString:SetText(L["Player"] .. " " .. categoryName)
	
	local teamFontString = legendFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	teamFontString:SetPoint("LEFT", playerFontString, "RIGHT", 10, 0) 
	teamFontString:SetTextColor(1, 1, 0, 1)
	teamFontString:SetText(L["Team"] .. " " .. categoryName)
	
	
	return legendFrame
end

--[[
 -	Builds the Pie chart used by the Kills, Damage, and Healing widgets.
 -
 -	@param Frame parent
 -	@param table values keyed on rank (int) whose value is a double. The values should add to 100.
 -	@return Frame
--]]
function createPieChart(parent, values)
	local colors = { ["ranked"] = {0.0,1.0,0.0}, ["unranked"] = {1.0,0.0,0.0} }
	if not parent.pieChart then
		-- Only create this travesty of justice once and reuse it.
		local Graph = LibStub("LibGraph-2.0")
		parent.pieChart = Graph:CreateGraphPieChart("TestPieChart", parent, "LEFT", "RIGHT",350, -10, 100, 100)
		createPieLegendFrame(nil, parent.pieChart, "TOP", "BOTTOM", colors)	
	else
		parent.pieChart:ResetPie()
	end
	
	for rank, value in pairs (values) do
		parent.pieChart:AddPie(value, colors[rank])
	end
	
	return parent.pieChart
end

--[[
 -	Generates a Line chart using LibGraph-2.0. This accepts an array or table
 -	whatever the hell you call it that is keyed from a 0 based integer index,
 -	and whose value is a ranking 0 - 5, where 0 means unranked. 
 -
 -	The auto scale feature is a little wonky so we are going to set axis manually.
 -	Also we will scale all data to fit a y-axis of 1. This needs to happen or the
 -	chart library will get furious.
 -
 -	@param Frame parent
 -	@param table playerValues
 -	@param table teamValues
 -	@param string categoryName
 -	@return Frame
--]]
function createLineChart(parent, playerValues, teamValues, categoryName)
	local scaledPlayer = {}
	local scaledTeam = {}
	local highest = 0
	
	-- Get highest value so we can scale
	for _, value in spairs(playerValues, function(t,a,b) 
			return t[b][2] < t[a][2]
		end) 
	do
		highest = value[2]
		break
	end
	for _, value in spairs(teamValues, function(t,a,b) 
			return t[b][2] < t[a][2]
		end) 
	do
		if value[2] > highest then
			highest = value[2]
		end
		break
	end
	
	for _, value in pairs(playerValues) do
		local scaledValue = { value[1], value[2] / highest }
		table.insert(scaledPlayer, { value[1], value[2] / highest })
	end
	
	for _, value in pairs(teamValues) do
		local scaledValue = { value[1], value[2] / highest }
		table.insert(scaledTeam, { value[1], value[2] / highest })
	end

	-- Apparently no way to delete frames, so we need to reuse them.
	if not parent.lineChart then
		local Graph = LibStub("LibGraph-2.0")
		parent.lineChart = Graph:CreateGraphLine("TestLineGraph", parent, "LEFT", "RIGHT", 170, -10, 150, 75)
		parent.lineChart:SetXAxis(0, tablelength(playerValues))
		parent.lineChart:SetYAxis(0, 1)
		parent.lineChart:SetGridColor({0.5,0.5,0.5,0.2})
		parent.lineChart:SetYLabels(false, false)
		parent.lineChart:SetAxisDrawing(true, true)
		parent.lineChart:SetAxisColor({1.0,1.0,1.0,1.0})
		createLineLegend(nil, parent.lineChart, "TOP", "BOTTOM", categoryName)
	else
		parent.lineChart:ResetData()
	end

	parent.lineChart:AddDataSeries(scaledPlayer, {0.0,1.0,0.0,0.8})
	parent.lineChart:AddDataSeries(scaledTeam, {1.0,1.0,0.0,0.8})
	return parent.lineChart
end

--[[
 -	Shows the Match History screen. This will use LibgGraph-2.0 to build charts and
 -	various worthless UI elemetns.
 -
 -	@param MatchHistoryService
--]]
function displayMatchHistory(history)
	local records = history:GetAll()
	
	if not records or tablelength(records) == 0 then 
		print(L["NoDataRecorded"])
		return 
	end
	
	local historyTitle = string.gsub(L["HistoryTitle"], "#COUNT#", tablelength(records))
	matchHistoryFrame.descriptionFontString:SetText(historyTitle)
	
	local teamWins = history:GetTotal("didPlayerWin")
	local teamLosses = tablelength(records) - teamWins
	local winRate = round(history:GetRatio(
		function(r) return r.didPlayerWin and 1 or 0 end, 
		function(r) return r.didPlayerWin and 0 or 1 end
	), 2)
	local teamSummary = teamWins .. " " .. L["Wins"] .. " - " .. teamLosses .. " " .. L["Losses"] .. " (" .. winRate .. "% " .. L["WinRate"] .. ")"
	matchHistoryFrame.teamSummaryFontString:SetText(teamSummary)
	
	local playerKills = history:GetTotal("kills")
	local playerDeaths = history:GetTotal("deaths")
	local playerKillRatio = round(history:GetRatio(
		function(r) return r.kills end, 
		function(r) return r.deaths end		
	), 2)
	local playerSummary = playerKills .. " " .. L["Kills"] .. " - " .. playerDeaths .. " " .. L["Deaths"] .. " (" .. playerKillRatio .. "% " .. L["KillDeathRatio"] .. ")"
	matchHistoryFrame.playerSummaryFontString:SetText(playerSummary)
	
	local pieChartDataKills = history:GetRankPercentBreakdownData("rank")
	local lineChartDataPlayerKills = history:GetTrendData("kills")
	local lineChartDataAvgTeamKills = history:GetTrendData("avgTeamKills")
	createPieChart(matchHistoryFrame.killsCrestFrame, pieChartDataKills)
	createLineChart(matchHistoryFrame.killsCrestFrame, lineChartDataPlayerKills, lineChartDataAvgTeamKills, L["Kills"])
	
	local lineChartDataDeaths = history:GetTrendData("deaths")
	local lineChartDataAvgTeamDeaths = history:GetTrendData("avgTeamDeaths")
	createLineChart(matchHistoryFrame.noDeathCrestFrame, lineChartDataDeaths, lineChartDataAvgTeamDeaths, L["Deaths"])

	local pieChartDataDamage = history:GetRankPercentBreakdownData("damageRank")
	local lineChartDataDamage = history:GetTrendData("damage")
	local lineChartDataAvgTeamDamage = history:GetTrendData("avgTeamDamage")
	createPieChart(matchHistoryFrame.topDamageCrestFrame, pieChartDataDamage)
	createLineChart(matchHistoryFrame.topDamageCrestFrame, lineChartDataDamage, lineChartDataAvgTeamDamage, L["Damage"])
	
	local pieChartDataHealing = history:GetRankPercentBreakdownData("healingRank")
	local lineChartDataHealing = history:GetTrendData("healing")
	local lineChartDataAvgTeamHealing = history:GetTrendData("avgTeamHealing")
	createPieChart(matchHistoryFrame.topHealingCrestFrame, pieChartDataHealing)
	createLineChart(matchHistoryFrame.topHealingCrestFrame, lineChartDataHealing, lineChartDataAvgTeamHealing, L["Healing"])

	local totalKills = L["Total"] .. ": " .. prettyNumber(history:GetTotal("kills")) .. "\n"
	local averageKills = L["Average"] .. ": " .. prettyNumber(history:GetAverage("kills")) .. "\n"
	local highestKills = L["Highest"] .. ": " .. prettyNumber(history:GetHighestValue("kills")) .. "\n"
	local teamContributionKills = L["TeamContribution"] .. ": " .. prettyNumber(history:GetAverage("playerPercentKillsOfTeam")) .. "%\n"
	local averageRankKills = L["AverageRank"] .. ": " .. history:GetAverage("rank")
	matchHistoryFrame.killsCrestFrame.summaryFontString:SetText(totalKills .. averageKills .. highestKills .. teamContributionKills .. averageRankKills)
	
	local totalDeaths = L["Total"] .. ": " .. prettyNumber(history:GetTotal("deaths")) .. "\n"
	local averageDeaths = L["Average"] .. ": " .. prettyNumber(history:GetAverage("deaths")) .. "\n"
	local highestDeaths = L["Highest"] .. ": " .. prettyNumber(history:GetHighestValue("deaths")) .. "\n"
	matchHistoryFrame.noDeathCrestFrame.summaryFontString:SetText(totalDeaths .. averageDeaths .. highestDeaths)
	
	local totalDamage = L["Total"] .. ": " .. prettyNumber(history:GetTotal("damage")) .. "\n"
	local averageDamage = L["Average"] .. ": " .. prettyNumber(history:GetAverage("damage")) .. "\n"
	local highestDamage = L["Highest"] .. ": " .. prettyNumber(history:GetHighestValue("damage")) .. "\n"
	local teamContributionDamage = L["TeamContribution"] .. ": " .. prettyNumber(history:GetAverage("playerPercentDamageOfTeam")) .. "%\n"
	local averageRankDamage = L["AverageRank"] .. ": " .. history:GetAverage("damageRank")
	matchHistoryFrame.topDamageCrestFrame.summaryFontString:SetText(totalDamage .. averageDamage .. highestDamage .. teamContributionDamage .. averageRankDamage)
	
	local totalHealing = L["Total"] .. ": " .. prettyNumber(history:GetTotal("healing")) .. "\n"
	local averageHealing = L["Average"] .. ": " .. prettyNumber(history:GetAverage("healing")) .. "\n"
	local highestHealing = L["Highest"] .. ": " .. prettyNumber(history:GetHighestValue("healing")) .. "\n"
	local teamContributionHealing = L["TeamContribution"] .. ": " .. prettyNumber(history:GetAverage("playerPercentHealingOfTeam")) .. "%\n"
	local averageRankHealing = L["AverageRank"] .. ": " .. history:GetAverage("healingRank")
	matchHistoryFrame.topHealingCrestFrame.summaryFontString:SetText(totalHealing .. averageHealing .. highestHealing .. teamContributionHealing .. averageRankHealing)
	
	local killRank = math.floor(history:GetAverage("rank"))
	if killRank > 0 and killRank <= 5 then
		matchHistoryFrame.killsCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\DerpRank" .. killRank)
	else
		matchHistoryFrame.killsCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
	end

	if history:GetAverage("deaths") <= 1 then
		matchHistoryFrame.noDeathCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\SippieCupRank")
	else
		matchHistoryFrame.noDeathCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
	end
	
	local damageRank = math.floor(history:GetAverage("damageRank"))
	if damageRank > 0 and damageRank <= 5 then
		matchHistoryFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\WheelchairRank" .. damageRank)
	else
		matchHistoryFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
	end
	
	local healingRank = math.floor(history:GetAverage("healingRank"))
	if healingRank > 0 and healingRank <= 5 then
		matchHistoryFrame.topHealingCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\WheelchairRank" .. healingRank)
	else
		matchHistoryFrame.topHealingCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
	end
	
	local topSpecKills = history:GetTopSpecs("kills")
	updateTopSpecs(matchHistoryFrame.topSpecKills, topSpecKills, false)
	
	local topSpecDamage = history:GetTopSpecs("damage")
	updateTopSpecs(matchHistoryFrame.topSpecDamage, topSpecDamage, false)
	
	local topSpecHealing = history:GetTopSpecs("healing")
	updateTopSpecs(matchHistoryFrame.topSpecHealing, topSpecHealing, false)
	
	local topSpecMostSeen = history:GetTopSpecs("all")
	updateTopSpecs(matchHistoryFrame.topSpecMostSeen, topSpecMostSeen, true)
	
	local mapData = history:GetMapData()
	updateBattlegroundStats(matchHistoryFrame.battlegroundStats, mapData)
	
	matchHistoryFrame:Show()
end

--[[
 -	Adds MongoMon UI elements to nameplates.
 -
 -	1) Kill - Death score
 -	2) Damage Rank
 -	3) Killing Blows Rank
 -
 -	@param table Player object
 -	@param Frame plateFrame
--]]
function addMongoMonToNameplate(player, plateFrame)
	-- Create Fontstring for Kill - Death score
	local scoreFontString = plateFrame:CreateFontString()
	scoreFontString:SetFont(_G.NumberFont_Shadow_Small:GetFont(), 15, "THICKOUTLINE")
	if player.enemy then
		scoreFontString:SetTextColor(1, 0.1, 0.1, 0.75)
	else
		scoreFontString:SetTextColor(0, 1, 0, 1)
	end
	scoreFontString:SetPoint("CENTER", 0, 35)
	scoreFontString:SetText(player.kills .. " - " .. player.deaths)
	
	local rankTexture = plateFrame:CreateTexture()
	rankTexture:SetWidth(32)
	rankTexture:SetHeight(32)
	rankTexture:SetPoint("TOPLEFT", plateFrame, "TOPLEFT", 0, 25)
	rankTexture:Hide()

	if player.rank > 0 and player.rank <= MAX_NAMEPLATE_RANK and player.kills > 0 then
		rankTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\DerpRank" .. player.rank)
		rankTexture:Show()
	end

	-- Top Damage Texture	
	local topDamageFrame = CreateFrame("FRAME", nil, plateFrame)
	topDamageFrame:SetPoint("TOPRIGHT", plateFrame, "TOPRIGHT", -10, 25)
	topDamageFrame:SetWidth(34)
	topDamageFrame:SetHeight(34)
	
	-- Top Damage Texture
	topDamageFrame.topDamageTexture = topDamageFrame:CreateTexture()
	topDamageFrame.topDamageTexture:SetPoint("CENTER")
	topDamageFrame.topDamageTexture:SetAllPoints()
	topDamageFrame:Hide()
	topDamageFrame.topDamageTexture:Hide()
	
	if player.damage > 0 and (player.damageRank >= 1 and player.damageRank <= MAX_NAMEPLATE_RANK) then
		topDamageFrame:Show()
		topDamageFrame.topDamageTexture:Show()
		topDamageFrame.topDamageTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\WheelchairRank" .. player.damageRank)
	end	

	plateFrame.MM = {
		scoreFontString = scoreFontString,
		rankTexture = rankTexture,
		topDamageFrame = topDamageFrame
	}
end

--[[
 -	Updates an existing plateFrame provided by LibNameplateRegistry. We will determine if
 -	we should make any changes to the MongoMon elements:
 -	1) Kill - Death score
 -	2) Damage Rank
 -	3) Killing Blows Rank
 -
 -	@param table Player object
 -	@param Frame plateFrame
--]]
function updateNameplate(player, plateFrame)
	if not player then return end
	if not plateFrame or not plateFrame.MM or not plateFrame.MM.topDamageFrame or not plateFrame.MM.rankTexture or not plateFrame.MM.scoreFontString then return end
	
	-- Player Kill / Death score
	plateFrame.MM.scoreFontString:Show()
	plateFrame.MM.scoreFontString:SetText(player.kills .. " - " .. player.deaths )
	
	-- Killing Blow Icon
	if player.rank > 0 and player.rank <= MAX_NAMEPLATE_RANK and player.kills > 0 then
		plateFrame.MM.rankTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\DerpRank" .. player.rank)
		plateFrame.MM.rankTexture:Show()
	else
		plateFrame.MM.rankTexture:Hide()
	end
	
	-- Top Damage Icon
	if player.damage > 0 and (player.damageRank >= 1 and player.damageRank <= MAX_NAMEPLATE_RANK) then
		plateFrame.MM.topDamageFrame:Show()
		plateFrame.MM.topDamageFrame.topDamageTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\WheelchairRank" .. player.damageRank)
		plateFrame.MM.topDamageFrame.topDamageTexture:Show()
	else
		plateFrame.MM.topDamageFrame.topDamageTexture:Hide()
		plateFrame.MM.topDamageFrame:Hide()
	end
end

--[[
 -	Gets the approprivate flavor text for the After Action report.
 - 	Uses the provided MatchHistoryRecord to determine which one to use.
 -
 -	@param MatchHistoryRecord
 -	@return string
--]]
function getFlavorText(record)
	if record.kills > record.teamKills then 
		return L["FlavorBestPlayerKillsTeam"]
	end 
	
	if record.damage > record.teamDamage then
		return L["FlavorBestPlayerDamageTeam"]
	end

	if record.healing > record.teamHealing then
		return L["FlavorBestPlayerHealingTeam"]
	end
	
	if record.kills > record.enemyKills then 
		return L["FlavorBestPlayerKillsEnemy"]
	end 
	
	if record.damage > record.enemyDamage then
		return L["FlavorBestPlayerDamageEnemy"]
	end

	if record.healing > record.enemyHealing then
		return L["FlavorBestPlayerHealingEnemy"]
	end
	
	-- Possible flavor text suggestions. Will be random depending on outcome of game and player
	-- performance.
	local flavor
	if (record.rank <= MAX_PLAYER_RANK or record.damageRank <= MAX_PLAYER_RANK or record.healingRank <= MAX_PLAYER_RANK) and record.deaths <= 2 then -- Good player
		flavor = { L["FlavorGoodPlayer"], L["FlavorGoodPlayer2"], L["FlavorGoodPlayer3"], L["FlavorGoodPlayer4"], L["FlavorGoodPlayer5"], L["FlavorGoodPlayer6"] }
		if not record.didPlayerWin then
			flavor = { L["FlavorGoodPlayerBadTeam"], L["FlavorGoodPlayerBadTeam2"] }
		end
	else -- Bad player
		flavor = { L["FlavorBadPlayer"], L["FlavorBadPlayer2"], L["FlavorBadPlayer3"], L["FlavorBadPlayer4"], L["FlavorBadPlayer5"] }
		if not record.didPlayerWin then
			table.insert(flavor, L["FlavorBadPlayerBadTeam"])
		else
			table.insert(flavor, L["FlavorBadPlayerGoodTeam"])
		end
		
		if record.deaths == 0 then
			table.insert(flavor, L["FlavorBadPlayerNoDeaths"])
		end
	end

	local index = math.random(1, table.getn(flavor))
	return flavor[index]
end

--[[
 -	Prepares the after action report. This screen will display the final icons for
 -	kills, damage/healing, and no deaths. It will also summarize briefly the player
 -	stats, performance of his team and some flavor text.
 - 
 -	Display killing blow rank
 -	Display damage rank
 -	Display no death
 -	Get total team damage
 -	Get total enemy damage
 -	Get player percentage damage of team
 -	Get percent team damage vs enemy
 -	Get percent team healing vs enemy 
 -
 -	@param MatchHistoryRecord
--]]
function displayAfterActionReport(record)
	if record ~= nil then
		score:Hide()
		
		local teamVsEnemyDamage = ""
		local teamVsEnemyHealing = ""
		
		if record.enemyDamage > record.teamDamage then
			local damageDifference = string.format("%.2f", (record.enemyDamage - record.teamDamage) / record.teamDamage * 100)
			teamVsEnemyDamage = string.gsub(L["ResultEnemyTeamOutDamaged"], "#DAMAGE#", damageDifference)
		else
			local damageDifference = string.format("%.2f", (record.teamDamage - record.enemyDamage) / record.enemyDamage * 100)
			teamVsEnemyDamage = string.gsub(L["ResultPlayerTeamOutDamaged"], "#DAMAGE#", damageDifference)
		end
		
		if record.enemyHealing > record.teamHealing then
			local healingDifference = string.format("%.2f", (record.enemyHealing - record.teamHealing) / record.teamHealing * 100)
			teamVsEnemyHealing = string.gsub(L["ResultEnemyTeamOutHealed"], "#HEALING#", healingDifference)
		else
			local healingDifference = string.format("%.2f", (record.teamHealing - record.enemyHealing) / record.enemyHealing * 100)
			teamVsEnemyHealing = string.gsub(L["ResultPlayerTeamOutHealed"], "#HEALING#", healingDifference)
		end
		
		afterActionFrame.killsCrestFrame.subFontString:SetText(L["Rank"] .. " " .. record.rank)
		if record.isHealer then
			afterActionFrame.topDamageCrestFrame.subFontString:SetText(L["Rank"] .. " " .. record.healingRank)
		else
			afterActionFrame.topDamageCrestFrame.subFontString:SetText(L["Rank"] .. " " .. record.damageRank)
		end

		local playerReport =  string.gsub(L["ResultPlayerPercentTeamDamage"], "#DAMAGE#", record.playerPercentDamageOfTeam)
		playerReport = string.gsub(playerReport, "#KILLS#", record.playerPercentKillsOfTeam)
		playerReport = playerReport .. " "  .. string.gsub(L["ResultPlayerPercentTeamHealing"], "#HEALING#", record.playerPercentHealingOfTeam)
		afterActionFrame.reportPlayerFontString:SetText(playerReport)
		
		local teamReport = string.gsub(L["ResultTeamKillDeath"], "#KILLS#", record.teamKills)
		teamReport = string.gsub(teamReport, "#DEATHS#", record.teamDeaths)
		teamReport = teamReport .. " " .. teamVsEnemyDamage
		teamReport = teamReport .. " " .. teamVsEnemyHealing
		afterActionFrame.reportTeamFontString:SetText(teamReport)
		
		local flavor = getFlavorText(record)
		afterActionFrame.flavorFontString:SetText(flavor)
		
		if record.rank > 0 and record.rank <= MAX_PLAYER_RANK and record.kills > 0 then
			afterActionFrame.killsCrestTexture:SetAlpha(1)
			afterActionFrame.killsCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\DerpRank" .. record.rank)
		else
			afterActionFrame.killsCrestTexture:SetAlpha(ALPHA_VALUE)
			afterActionFrame.killsCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
		end
		afterActionFrame.killsCrestFrame.fontString:SetText(record.kills .. " " .. L["Kills"])
		
		if record.isHealer then
			if record.healingRank >= 1 and record.healingRank <= MAX_PLAYER_RANK and record.healing > 0 then
				afterActionFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\WheelchairRank" .. record.healingRank)
				afterActionFrame.topDamageCrestTexture:SetAlpha(1)
			else 
				afterActionFrame.topDamageCrestTexture:SetAlpha(ALPHA_VALUE)
				afterActionFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
			end	
			afterActionFrame.topDamageCrestFrame.fontString:SetText(prettyNumber(record.healing) .. " " .. L["Healing"])
		else
			if record.damageRank >= 1 and record.damageRank <= MAX_PLAYER_RANK and record.damage > 0 then
				afterActionFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\WheelchairRank" .. record.damageRank)
				afterActionFrame.topDamageCrestTexture:SetAlpha(1)
			else 
				afterActionFrame.topDamageCrestTexture:SetAlpha(ALPHA_VALUE)
				afterActionFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
			end	
			afterActionFrame.topDamageCrestFrame.fontString:SetText(prettyNumber(record.damage) .. " " .. L["Damage"])
		end
		
		if record.deaths == 0 then
			afterActionFrame.noDeathCrestTexture:SetAlpha(1)
			afterActionFrame.noDeathCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\SippieCupRank")
		else
			afterActionFrame.noDeathCrestTexture:SetAlpha(ALPHA_VALUE)
			afterActionFrame.noDeathCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
		end
		afterActionFrame.noDeathCrestFrame.fontString:SetText(record.deaths .. " " .. L["Deaths"])
	
		afterActionFrame:Show()
	end
end

--[[
 -	Wow UI frame creation. This contains the Personal scoreboard frame
 -	as well as the Match History UI panel that shows a detailed list of
 -	past BG matches as well as a summary of stats like win percentage
 -	and yoru overall derp rank.
--]]

-- So everything in this game that is event related must be part of a widget.
frame = CreateFrame("FRAME", "MongoMonFrame")

-- Scoreboard frame that will display Kill - Death in BG's
score = CreateFrame("FRAME", "MongoMonScoreFrame")
score:SetMovable(true)
score:EnableMouse(true)
score:RegisterForDrag("LeftButton")
score:SetScript("OnDragStart", score.StartMoving)
score:SetScript("OnDragStop", score.StopMovingOrSizing)
score:SetPoint("CENTER")
score:SetWidth(200)
score:SetHeight(110)
score:SetBackdrop(BACKDROP_BAD)
score:SetScript("OnMouseUp", function(self, button) -- Toggle the actual bg scoreboard when you press this.
	LibStub("LibButtonGlow-1.0").ShowOverlayGlow(score)
	LibStub("LibButtonGlow-1.0").HideOverlayGlow(score)	
	ToggleWorldStateScoreFrame()
end)

-- Gradient background overlay, this makes the empty icons look better
score.backgroundFrame = CreateFrame("FRAME", nil, score)
score.backgroundFrame:SetPoint("CENTER", score, "CENTER", 0, 0)
score.backgroundFrame:SetSize(185, 90)
score.backgroundTexture = score.backgroundFrame:CreateTexture()
score.backgroundTexture:SetAlpha(0.15)
score.backgroundTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\ScoreboardBackground")
score.backgroundTexture:SetAllPoints()

-- Top row icon: Killing Blow rank
score.rankFrame = CreateFrame("FRAME", nil, score)
score.rankFrame:SetPoint("TOPLEFT", score, "TOPLEFT", 25, -12)
score.rankFrame:SetSize(40, 40)
score.rankTexture = score.rankFrame:CreateTexture()
score.rankTexture:SetAlpha(ALPHA_VALUE)
score.rankTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
score.rankTexture:SetAllPoints()
score.rankTitle = score:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
score.rankTitle:SetPoint("TOPLEFT", score, "TOPLEFT", 34, -54) 
score.rankTitle:SetText(L["RankTitle"])

-- Top row icon: Top Damage Icon
score.topDamageFrame = CreateFrame("FRAME", nil, score)
score.topDamageFrame:SetPoint("TOP", score, "TOP", 0, -12)
score.topDamageFrame:SetSize(40, 40)
score.topDamageTexture = score.topDamageFrame:CreateTexture()
score.topDamageTexture:SetAlpha(ALPHA_VALUE)
score.topDamageTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
score.topDamageTexture:SetAllPoints()
score.topDamageTitle = score:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
score.topDamageTitle:SetPoint("TOP", score, "TOP", 2, -54)
score.topDamageTitle:SetText(L["TopDamageTitle"])

-- Top row icon: No Death
score.noDeathFrame = CreateFrame("FRAME", nil, score)
score.noDeathFrame:SetPoint("TOPRIGHT", score, "TOPRIGHT", -25, -12)
score.noDeathFrame:SetSize(40, 40)
score.noDeathTexture = score.noDeathFrame:CreateTexture()
score.noDeathTexture:SetAlpha(ALPHA_VALUE)
score.noDeathTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
score.noDeathTexture:SetAllPoints()
score.noDeathTitle = score:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
score.noDeathTitle:SetPoint("TOPRIGHT", score, "TOPRIGHT", -24, -54) 
score.noDeathTitle:SetText(L["NoDeathTitle"])

-- Title 
score.title = score:CreateFontString(nil, "ARTWORK", "GameFontHighlightOutline")
score.title:SetPoint("BOTTOM", score, "TOP", 0, -5)
score.title:SetText(L["Title"])

-- Kill - Death Fontstring
score.fontString = score:CreateFontString(nil, "ARTWORK", "GameFontNormal")
score.fontString:SetPoint("BOTTOM", score, 0, 30) 

-- Damage fontstring
score.damageFontString = score:CreateFontString(nil, "ARTWORK", "GameFontNormal")
score.damageFontString:SetPoint("BOTTOM", score, 0, 15)

--[[
 -	The After Action Report.
 -
 -	This section builds the UI banner that is displayed on the screen after a battleground that
 -	summarizes our hero's glorious BG achievements.
--]]

-- Primary container and background
afterActionFrame = CreateFrame("FRAME", "MongoMonAfterActionFrame")
afterActionFrame:SetFrameStrata("HIGH") -- Appear above annoying junk
afterActionFrame:SetFrameLevel(10) -- Appear above annoying junk
afterActionFrame:SetPoint("TOP", nil, "TOP", 0, -20)
afterActionFrame:SetWidth(450)
afterActionFrame:SetHeight(260)
afterActionFrame:SetBackdrop(BACKDROP_GREEN)
afterActionFrame:SetMovable(true)
afterActionFrame:EnableMouse(true)
afterActionFrame:RegisterForDrag("LeftButton")
afterActionFrame:SetScript("OnDragStart", afterActionFrame.StartMoving)
afterActionFrame:SetScript("OnDragStop", afterActionFrame.StopMovingOrSizing)
afterActionFrame:SetScript("OnMouseUp", function(self, button)
	afterActionFrame:Hide()
end)

afterActionFrame.title = afterActionFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightOutline")
afterActionFrame.title:SetPoint("BOTTOM", afterActionFrame, "TOP", 0, -5)
afterActionFrame.title:SetText(L["Title"])

afterActionFrame.backgroundFrame = CreateFrame("FRAME", nil, afterActionFrame)
afterActionFrame.backgroundFrame:SetFrameLevel(0)
afterActionFrame.backgroundFrame:SetPoint("CENTER", afterActionFrame, "CENTER", 0, 0)
afterActionFrame.backgroundFrame:SetSize(435, 240)
afterActionFrame.backgroundTexture = afterActionFrame.backgroundFrame:CreateTexture()
afterActionFrame.backgroundTexture:SetAlpha(0.55)
afterActionFrame.backgroundTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\ScoreboardBackground")
afterActionFrame.backgroundTexture:SetAllPoints()

afterActionFrame.flavorFontString = afterActionFrame:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
afterActionFrame.flavorFontString:SetPoint("BOTTOM", afterActionFrame, "BOTTOM", 0, 50) 
afterActionFrame.flavorFontString:SetJustifyH("LEFT")
afterActionFrame.flavorFontString:SetWidth(380)

afterActionFrame.reportTeamFontString = afterActionFrame:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
afterActionFrame.reportTeamFontString:SetPoint("BOTTOM", afterActionFrame.flavorFontString, "TOP", 0, 10) 
afterActionFrame.reportTeamFontString:SetJustifyH("LEFT")
afterActionFrame.reportTeamFontString:SetWidth(380)

afterActionFrame.reportPlayerFontString = afterActionFrame:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
afterActionFrame.reportPlayerFontString:SetPoint("BOTTOM", afterActionFrame.reportTeamFontString, "TOP", 0, 10) 
afterActionFrame.reportPlayerFontString:SetJustifyH("LEFT")
afterActionFrame.reportPlayerFontString:SetWidth(380)

-- Top Killing Blows
afterActionFrame.killsCrestFrame = CreateFrame("FRAME", nil, afterActionFrame)
afterActionFrame.killsCrestFrame:SetWidth(85)
afterActionFrame.killsCrestFrame:SetHeight(85)
afterActionFrame.killsCrestFrame:SetPoint("TOPLEFT", afterActionFrame, "TOPLEFT", 50, -10)

afterActionFrame.killsCrestTexture = afterActionFrame.killsCrestFrame:CreateTexture()
afterActionFrame.killsCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
afterActionFrame.killsCrestTexture:SetAllPoints()

afterActionFrame.killsCrestFrame.fontString = afterActionFrame.killsCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
afterActionFrame.killsCrestFrame.fontString:SetPoint("TOP", afterActionFrame.killsCrestFrame, "BOTTOM", 0, 0) 
afterActionFrame.killsCrestFrame.fontString:SetJustifyH("CENTER")

afterActionFrame.killsCrestFrame.subFontString = afterActionFrame.killsCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
afterActionFrame.killsCrestFrame.subFontString:SetPoint("TOP", afterActionFrame.killsCrestFrame.fontString, "BOTTOM", 0, -3) 
afterActionFrame.killsCrestFrame.subFontString:SetJustifyH("CENTER")

-- Top Damage
afterActionFrame.topDamageCrestFrame = CreateFrame("FRAME", nil, afterActionFrame)
afterActionFrame.topDamageCrestFrame:SetWidth(85)
afterActionFrame.topDamageCrestFrame:SetHeight(85)
afterActionFrame.topDamageCrestFrame:SetPoint("TOP", afterActionFrame, "TOP", 0, -10)

afterActionFrame.topDamageCrestTexture = afterActionFrame.topDamageCrestFrame:CreateTexture()
afterActionFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
afterActionFrame.topDamageCrestTexture:SetAllPoints()

afterActionFrame.topDamageCrestFrame.fontString = afterActionFrame.topDamageCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
afterActionFrame.topDamageCrestFrame.fontString:SetPoint("TOP", afterActionFrame.topDamageCrestFrame, "BOTTOM", 0, 0) 
afterActionFrame.topDamageCrestFrame.fontString:SetJustifyH("CENTER")

afterActionFrame.topDamageCrestFrame.subFontString = afterActionFrame.topDamageCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
afterActionFrame.topDamageCrestFrame.subFontString:SetPoint("TOP", afterActionFrame.topDamageCrestFrame.fontString, "BOTTOM", 0, -3) 
afterActionFrame.topDamageCrestFrame.subFontString:SetJustifyH("CENTER")

-- No Deaths
afterActionFrame.noDeathCrestFrame = CreateFrame("FRAME", nil, afterActionFrame)
afterActionFrame.noDeathCrestFrame:SetWidth(85)
afterActionFrame.noDeathCrestFrame:SetHeight(85)
afterActionFrame.noDeathCrestFrame:SetPoint("TOPRIGHT", afterActionFrame, "TOPRIGHT", -50, -10)

afterActionFrame.noDeathCrestTexture = afterActionFrame.noDeathCrestFrame:CreateTexture()
afterActionFrame.noDeathCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
afterActionFrame.noDeathCrestTexture:SetAllPoints()

afterActionFrame.noDeathCrestFrame.fontString = afterActionFrame.noDeathCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
afterActionFrame.noDeathCrestFrame.fontString:SetPoint("TOP", afterActionFrame.noDeathCrestFrame, "BOTTOM", 0, 0) 
afterActionFrame.noDeathCrestFrame.fontString:SetJustifyH("CENTER")

afterActionFrame.footerFontString = afterActionFrame:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
afterActionFrame.footerFontString:SetPoint("BOTTOM", afterActionFrame, "BOTTOM", 0,22) 
afterActionFrame.footerFontString:SetText(L["TouchToClose"])

afterActionFrame.button = CreateFrame("BUTTON", nil, afterActionFrame, "UIPanelButtonTemplate");
afterActionFrame.button:SetPoint("BOTTOMRIGHT",afterActionFrame, "BOTTOMRIGHT", -15, 15)
afterActionFrame.button:RegisterForClicks("AnyUp")
afterActionFrame.button:SetSize(125, 25)
afterActionFrame.button:SetText("Send to chat")
afterActionFrame.button:SetScript("OnClick", function(self) 
	if not self.lastReportTime or self.lastReportTime + SEND_TO_CHAT_COOLDOWN < time() then
		self.lastReportTime = time()
		local version = GetAddOnMetadata("MongoMon", "Version")
		local text = "[" .. L["Title"] .. " v" .. version .. "] " .. self:GetParent().reportTeamFontString:GetText()
		SendChatMessage(text, "INSTANCE_CHAT")
	end
end)

--[[
 -	Match History Statistics
 -
 -	This screen provides statistics over the span of multiple battlegrounds.
--]]

matchHistoryFrame = CreateFrame("FRAME", "MongoMonMatchHistoryFrame")
matchHistoryFrame:SetFrameStrata("HIGH")
matchHistoryFrame:SetPoint("CENTER", nil, "CENTER", 0, 0)
matchHistoryFrame:SetWidth(830)
matchHistoryFrame:SetHeight(685)
matchHistoryFrame:SetBackdrop(BACKDROP_BAD)
matchHistoryFrame:SetMovable(true)
matchHistoryFrame:EnableMouse(true)
matchHistoryFrame:RegisterForDrag("LeftButton")
matchHistoryFrame:SetScript("OnDragStart", matchHistoryFrame.StartMoving)
matchHistoryFrame:SetScript("OnDragStop", matchHistoryFrame.StopMovingOrSizing)
matchHistoryFrame:SetScript("OnMouseUp", function(self, button)
	matchHistoryFrame:Hide()
end)

matchHistoryFrame.title = matchHistoryFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightOutline")
matchHistoryFrame.title:SetPoint("BOTTOM", matchHistoryFrame, "TOP", 0, -5)
matchHistoryFrame.title:SetText(L["Title"])

matchHistoryFrame.backgroundFrame = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.backgroundFrame:SetFrameLevel(0)
matchHistoryFrame.backgroundFrame:SetPoint("CENTER", matchHistoryFrame, "CENTER", 0, 0)
matchHistoryFrame.backgroundFrame:SetSize(820, 675)
matchHistoryFrame.backgroundTexture = matchHistoryFrame.backgroundFrame:CreateTexture()
matchHistoryFrame.backgroundTexture:SetAlpha(ALPHA_VALUE)
matchHistoryFrame.backgroundTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\ScoreboardBackground")
matchHistoryFrame.backgroundTexture:SetAllPoints()

matchHistoryFrame.descriptionFontString = matchHistoryFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.descriptionFontString:SetPoint("TOP", matchHistoryFrame, "TOP", 0, -15) 

matchHistoryFrame.teamSummaryFontString = matchHistoryFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
matchHistoryFrame.teamSummaryFontString:SetPoint("TOP", matchHistoryFrame.descriptionFontString, "BOTTOM", 0, -5) 

matchHistoryFrame.playerSummaryFontString = matchHistoryFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
matchHistoryFrame.playerSummaryFontString:SetPoint("TOP", matchHistoryFrame.teamSummaryFontString, "BOTTOM", 0, -5) 

matchHistoryFrame.separatorFrame = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.separatorFrame:SetPoint("TOP", matchHistoryFrame.playerSummaryFontString, "BOTTOM", 0, -5)
matchHistoryFrame.separatorFrame:SetSize(750, 5)
matchHistoryFrame.separatorTexture = matchHistoryFrame.separatorFrame:CreateTexture()
matchHistoryFrame.separatorTexture:SetAlpha(ALPHA_VALUE)
matchHistoryFrame.separatorTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\ScoreboardBackground")
matchHistoryFrame.separatorTexture:SetAllPoints()

matchHistoryFrame.footerFontString = matchHistoryFrame:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
matchHistoryFrame.footerFontString:SetPoint("BOTTOM", matchHistoryFrame, "BOTTOM", 0, 11) 
matchHistoryFrame.footerFontString:SetText(L["TouchToClose"])

-- Killing Blows
matchHistoryFrame.killsCrestFrame = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.killsCrestFrame:SetWidth(85)
matchHistoryFrame.killsCrestFrame:SetHeight(85)
matchHistoryFrame.killsCrestFrame:SetPoint("TOPLEFT", matchHistoryFrame.separatorFrame, "BOTTOMLEFT", 10, -5)

matchHistoryFrame.killsCrestTexture = matchHistoryFrame.killsCrestFrame:CreateTexture()
matchHistoryFrame.killsCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
matchHistoryFrame.killsCrestTexture:SetAllPoints()

matchHistoryFrame.killsCrestFrame.titleFontString = matchHistoryFrame.killsCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.killsCrestFrame.titleFontString:SetPoint("TOP", matchHistoryFrame.killsCrestFrame, "BOTTOM", 0, 0) 
matchHistoryFrame.killsCrestFrame.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.killsCrestFrame.titleFontString:SetText(L["RankTitle"])

matchHistoryFrame.killsCrestFrame.summaryFontString = matchHistoryFrame.killsCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
matchHistoryFrame.killsCrestFrame.summaryFontString:SetPoint("LEFT", matchHistoryFrame.killsCrestFrame, "RIGHT", 10, -10) 
matchHistoryFrame.killsCrestFrame.summaryFontString:SetJustifyH("LEFT")

-- Deaths
matchHistoryFrame.noDeathCrestFrame = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.noDeathCrestFrame:SetWidth(85)
matchHistoryFrame.noDeathCrestFrame:SetHeight(85)
matchHistoryFrame.noDeathCrestFrame:SetPoint("TOP", matchHistoryFrame.killsCrestFrame, "BOTTOM", 0, -30)

matchHistoryFrame.noDeathCrestTexture = matchHistoryFrame.noDeathCrestFrame:CreateTexture()
matchHistoryFrame.noDeathCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
matchHistoryFrame.noDeathCrestTexture:SetAllPoints()

matchHistoryFrame.noDeathCrestFrame.titleFontString = matchHistoryFrame.noDeathCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.noDeathCrestFrame.titleFontString:SetPoint("TOP", matchHistoryFrame.noDeathCrestFrame, "BOTTOM", 0, 0) 
matchHistoryFrame.noDeathCrestFrame.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.noDeathCrestFrame.titleFontString:SetText(L["NoDeathTitle"])

matchHistoryFrame.noDeathCrestFrame.summaryFontString = matchHistoryFrame.noDeathCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
matchHistoryFrame.noDeathCrestFrame.summaryFontString:SetPoint("LEFT", matchHistoryFrame.noDeathCrestFrame, "RIGHT", 10, 0) 
matchHistoryFrame.noDeathCrestFrame.summaryFontString:SetJustifyH("LEFT")

-- Damage
matchHistoryFrame.topDamageCrestFrame = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.topDamageCrestFrame:SetWidth(85)
matchHistoryFrame.topDamageCrestFrame:SetHeight(85)
matchHistoryFrame.topDamageCrestFrame:SetPoint("TOP", matchHistoryFrame.noDeathCrestFrame, "BOTTOM", 0, -30)

matchHistoryFrame.topDamageCrestTexture = matchHistoryFrame.topDamageCrestFrame:CreateTexture()
matchHistoryFrame.topDamageCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
matchHistoryFrame.topDamageCrestTexture:SetAllPoints()

matchHistoryFrame.topDamageCrestFrame.titleFontString = matchHistoryFrame.topDamageCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.topDamageCrestFrame.titleFontString:SetPoint("TOP", matchHistoryFrame.topDamageCrestFrame, "BOTTOM", 0, 0) 
matchHistoryFrame.topDamageCrestFrame.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.topDamageCrestFrame.titleFontString:SetText(L["TopDamageTitle"])

matchHistoryFrame.topDamageCrestFrame.summaryFontString = matchHistoryFrame.topDamageCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
matchHistoryFrame.topDamageCrestFrame.summaryFontString:SetPoint("LEFT", matchHistoryFrame.topDamageCrestFrame, "RIGHT", 10, -10) 
matchHistoryFrame.topDamageCrestFrame.summaryFontString:SetJustifyH("LEFT")

-- Healing
matchHistoryFrame.topHealingCrestFrame = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.topHealingCrestFrame:SetWidth(85)
matchHistoryFrame.topHealingCrestFrame:SetHeight(85)
matchHistoryFrame.topHealingCrestFrame:SetPoint("TOP", matchHistoryFrame.topDamageCrestFrame, "BOTTOM", 0, -30)

matchHistoryFrame.topHealingCrestTexture = matchHistoryFrame.topHealingCrestFrame:CreateTexture()
matchHistoryFrame.topHealingCrestTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\RankCrestGrayBackground")
matchHistoryFrame.topHealingCrestTexture:SetAllPoints()

matchHistoryFrame.topHealingCrestFrame.titleFontString = matchHistoryFrame.topHealingCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.topHealingCrestFrame.titleFontString:SetPoint("TOP", matchHistoryFrame.topHealingCrestFrame, "BOTTOM", 0, 0) 
matchHistoryFrame.topHealingCrestFrame.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.topHealingCrestFrame.titleFontString:SetText(L["TopHealingTitle"])

matchHistoryFrame.topHealingCrestFrame.summaryFontString = matchHistoryFrame.topHealingCrestFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
matchHistoryFrame.topHealingCrestFrame.summaryFontString:SetPoint("LEFT", matchHistoryFrame.topHealingCrestFrame, "RIGHT", 10, -10) 
matchHistoryFrame.topHealingCrestFrame.summaryFontString:SetJustifyH("LEFT")

-- Top Mongos: Kills (ranks 1 - 3)
matchHistoryFrame.topSpecKills = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.topSpecKills:SetWidth(175)
matchHistoryFrame.topSpecKills:SetHeight(100)
matchHistoryFrame.topSpecKills:SetPoint("LEFT", matchHistoryFrame.killsCrestFrame, "RIGHT", 480, -10)

matchHistoryFrame.topSpecKills.titleFontString = matchHistoryFrame.topSpecKills:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.topSpecKills.titleFontString:SetPoint("TOPLEFT", matchHistoryFrame.topSpecKills, "TOPLEFT", 0, 0) 
matchHistoryFrame.topSpecKills.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.topSpecKills.titleFontString:SetText(L["TopSpecs"] .. ": " .. L["Kills"])
createTopSpecs(matchHistoryFrame.topSpecKills)

-- Top Mongos: Most prevalent spec (ranks 1 - 3)
matchHistoryFrame.topSpecMostSeen = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.topSpecMostSeen:SetWidth(175)
matchHistoryFrame.topSpecMostSeen:SetHeight(100)
matchHistoryFrame.topSpecMostSeen:SetPoint("LEFT", matchHistoryFrame.noDeathCrestFrame, "RIGHT", 480, -10)

matchHistoryFrame.topSpecMostSeen.titleFontString = matchHistoryFrame.topSpecMostSeen:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.topSpecMostSeen.titleFontString:SetPoint("TOPLEFT", matchHistoryFrame.topSpecMostSeen, "TOPLEFT", 0, 0) 
matchHistoryFrame.topSpecMostSeen.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.topSpecMostSeen.titleFontString:SetText(L["TopSpecs"] .. ": " .. L["MostSeen"])
createTopSpecs(matchHistoryFrame.topSpecMostSeen)

-- Top Mongos: Damage (ranks 1 - 3)
matchHistoryFrame.topSpecDamage = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.topSpecDamage:SetWidth(175)
matchHistoryFrame.topSpecDamage:SetHeight(100)
matchHistoryFrame.topSpecDamage:SetPoint("LEFT", matchHistoryFrame.topDamageCrestFrame, "RIGHT", 480, -10)

matchHistoryFrame.topSpecDamage.titleFontString = matchHistoryFrame.topSpecDamage:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.topSpecDamage.titleFontString:SetPoint("TOPLEFT", matchHistoryFrame.topSpecDamage, "TOPLEFT", 0, 0) 
matchHistoryFrame.topSpecDamage.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.topSpecDamage.titleFontString:SetText(L["TopSpecs"] .. ": " .. L["Damage"])
createTopSpecs(matchHistoryFrame.topSpecDamage)

-- Top Mongos: Healing (ranks 1 - 3)
matchHistoryFrame.topSpecHealing = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.topSpecHealing:SetWidth(175)
matchHistoryFrame.topSpecHealing:SetHeight(100)
matchHistoryFrame.topSpecHealing:SetPoint("LEFT", matchHistoryFrame.topHealingCrestFrame, "RIGHT", 480, -10)

matchHistoryFrame.topSpecHealing.titleFontString = matchHistoryFrame.topSpecHealing:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
matchHistoryFrame.topSpecHealing.titleFontString:SetPoint("TOPLEFT", matchHistoryFrame.topSpecHealing, "TOPLEFT", 0, 0) 
matchHistoryFrame.topSpecHealing.titleFontString:SetJustifyH("CENTER")
matchHistoryFrame.topSpecHealing.titleFontString:SetText(L["TopSpecs"] .. ": " .. L["Healing"])
createTopSpecs(matchHistoryFrame.topSpecHealing)

-- Battleground map stats
matchHistoryFrame.battlegroundStats = CreateFrame("FRAME", nil, matchHistoryFrame)
matchHistoryFrame.battlegroundStats:SetWidth(830)
matchHistoryFrame.battlegroundStats:SetHeight(100)
matchHistoryFrame.battlegroundStats:SetPoint("TOPLEFT", matchHistoryFrame.topHealingCrestFrame, "BOTTOMLEFT", -50, -30)

matchHistoryFrame.battlegroundStats.separatorFrame = CreateFrame("FRAME", nil, matchHistoryFrame.battlegroundStats)
matchHistoryFrame.battlegroundStats.separatorFrame:SetPoint("TOP", matchHistoryFrame.battlegroundStats, "TOP", 0, 0)
matchHistoryFrame.battlegroundStats.separatorFrame:SetSize(750, 5)

matchHistoryFrame.battlegroundStats.separatorTexture = matchHistoryFrame.battlegroundStats.separatorFrame:CreateTexture()
matchHistoryFrame.battlegroundStats.separatorTexture:SetAlpha(ALPHA_VALUE)
matchHistoryFrame.battlegroundStats.separatorTexture:SetTexture("Interface\\AddOns\\MongoMon\\Res\\ScoreboardBackground")
matchHistoryFrame.battlegroundStats.separatorTexture:SetAllPoints()
createBattlegroundStats(matchHistoryFrame.battlegroundStats)

if DEBUG_ENABLED then -- Debug testing
	local records = {
		{
			["playerPercentHealingOfTeam"] = "10.52",
			["class"] = "Priest",
			["season"] = 0,
			["kills"] = 16,
			["time"] = 1470989198,
			["isRated"] = false,
			["enemyDeaths"] = 21,
			["playerPercentKillsOfTeam"] = "28.57",
			["mapId"] = 626,
			["damage"] = 5481216,
			["didPlayerWin"] = true,
			["deaths"] = 1,
			["enemyHealing"] = 14422486,
			["playerPercentDamageOfTeam"] = "19.21",
			["enemyKills"] = 27,
			["spec"] = "Shadow",
			["enemyDamage"] = 29023248,
			["teamKills"] = 21,
			["teamDeaths"] = 27,
			["teamHealing"] = 12247274,
			["name"] = "Fuhrbolg",
			["faction"] = "Horde",
			["teamDamage"] = 28528776,
			["healing"] = 2288715,
			["damageRank"] = 1,
			["healingRank"] = 5,
			["rank"] = 1,
			["avgTeamDamage"] = 1981216,
			["avgTeamHealing"] = 25088715,
			["avgTeamKills"] = 1.3,
			["avgTeamDeaths"] = 5.0,
			["isHealer"] = true,
			["players"] = {
				["healing"] = {
					["Priest-Holy"] = 2,
					["Priest-Discipline"] = 1

				},
				["damage"] = {
					["Warlock-Demonology"] = 1,
					["Demon Hunter-Havoc"] = 2
				},
				["kills"] = {
					["Demon Hunter-Havoc"] = 1,
					["Mage-Frost"] = 1,
					["Mage-Fire"] = 1
				},
				["all"] = {
					["Mage-Frost"] = 1,
					["Mage-Fire"] = 5,
					["Mage-Arcane"] = 4,
					["Priest-Shadow"] = 3,
					["Priest-Discipline"] = 2,
					["Priest-Holy"] = 3,
					["Demon Hunter-Havoc"] = 14,
					["Death Knight-Frost"] = 1,
					["Death Knight-Unholy"] = 1,
					["Death Knight-Blood"] = 5,
					["Rogue-Assassination"] = 3,
					["Rogue-Subtlety"] = 2,
					["Rogue-Outlaw"] = 5,
					["Warrior-Protection"] = 1,
					["Warrior-Arms"] = 3,
					["Warrior-Fury"] = 2,
					["Warlock-Demonology"] = 1,
					["Warlock-Affliction"] = 5,
					["Warlock-Destruction"] = 3
				}
			}
		},
		{
			["playerPercentHealingOfTeam"] = "10.52",
			["class"] = "Priest",
			["season"] = 0,
			["kills"] = 3,
			["time"] = 1470989199,
			["isRated"] = false,
			["enemyDeaths"] = 21,
			["playerPercentKillsOfTeam"] = "28.57",
			["mapId"] = 626,
			["damage"] = 15481216,
			["didPlayerWin"] = true,
			["deaths"] = 2,
			["enemyHealing"] = 14422486,
			["playerPercentDamageOfTeam"] = "19.21",
			["enemyKills"] = 27,
			["spec"] = "Shadow",
			["enemyDamage"] = 29023248,
			["teamKills"] = 21,
			["teamDeaths"] = 27,
			["teamHealing"] = 12247274,
			["name"] = "Fuhrbolg",
			["faction"] = "Horde",
			["teamDamage"] = 28528776,
			["healing"] = 1288715,
			["damageRank"] = 1,
			["healingRank"] = 9,
			["rank"] = 4,
			["avgTeamDamage"] = 4081216,
			["avgTeamHealing"] = 26288715,
			["avgTeamKills"] = 4.0,
			["avgTeamDeaths"] = 3.5,
			["isHealer"] = false
		},
		{
			["playerPercentHealingOfTeam"] = "10.52",
			["class"] = "Priest",
			["season"] = 0,
			["kills"] = 6,
			["time"] = 1470989200,
			["isRated"] = false,
			["enemyDeaths"] = 21,
			["playerPercentKillsOfTeam"] = "100.00",
			["mapId"] = 626,
			["damage"] = 3681216,
			["didPlayerWin"] = false,
			["deaths"] = 1,
			["enemyHealing"] = 14422486,
			["playerPercentDamageOfTeam"] = "19.21",
			["enemyKills"] = 27,
			["spec"] = "Shadow",
			["enemyDamage"] = 29023248,
			["teamKills"] = 21,
			["teamDeaths"] = 27,
			["teamHealing"] = 12247274,
			["healingRank"] = 17,
			["name"] = "Fuhrbolg",
			["faction"] = "Horde",
			["teamDamage"] = 28528776,
			["healing"] = 9288715,
			["damageRank"] = 1,
			["rank"] = 0,
			["avgTeamDamage"] = 3681216,
			["avgTeamHealing"] = 19288715,
			["avgTeamKills"] = 3.5,
			["avgTeamDeaths"] = 2.5,
			["isHealer"] = false
		}
	}

	local history = MatchHistoryService:Init(records)
	displayMatchHistory(history)
	displayAfterActionReport(records[1])
end